from pynecone import Subshell
from .login import Login
from .logout import Logout

class Auth(Subshell):

    def __init__(self):
        super().__init__("auth")

    def get_commands(self):
        return [Login(), Logout()]


from pynecone import Command
from .client import Client


class Find(Command):

    def __init__(self):
        super().__init__("find")

    def run(self, args):
        print(Client.create().get("items", {'public': 'true'}))



from http.server import BaseHTTPRequestHandler, HTTPServer
from urllib.parse import parse_qs
from webbrowser import open_new


class HTTPServerHandler(BaseHTTPRequestHandler):

    def __init__(self, request, address, server):
        super().__init__(request, address, server)

    def do_GET(self):
        if self.path.startswith('/auth?access_token='):
            self.server.access_token = parse_qs(self.path[6:])["access_token"][0]
            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.end_headers()
            self.wfile.write(bytes(
                "<html><head></head><h1>You may now close this window."
                + "</h1></html>", "utf-8"))
        elif self.path == '/':
            self.send_response(200)
            self.send_header('Content-type', 'text/html')
            self.end_headers()
            self.wfile.write(bytes("<html><head><script>window.location.href = '/auth?access_token=' + window.location.href.split('#')[1].split('&').filter(function(c) { return c.startsWith('access_token=')})[0].substring(13);</script></head><h1>You may now close this window."
                               + "</h1></html>", "utf-8"))
        return


class Authenticator:

    def __init__(self, a_id, a_uri, a_base):
        self._id = a_id
        self._uri = a_uri
        self._base = a_base

    def login(self):
        httpServer = HTTPServer(('localhost', 8080),
                                lambda req, address, server: HTTPServerHandler(req, address, server))

        open_new(self._base + '?client_id=' + self._id + '&redirect_uri=' + self._uri + '&response_type=token')

        httpServer.handle_request()
        httpServer.handle_request()

        return httpServer.access_token




from pynecone import Command
from .client import Client

class Login(Command):

    def __init__(self):
        super().__init__("login")

    def run(self, args):
        print("logging in")
        print(Client.create())



from .authenticator import Authenticator
import os
import keyring
import requests
from requests_toolbelt.utils import dump

from urllib.parse import urljoin

import dotenv
found_dotenv = dotenv.find_dotenv(usecwd=True)
print("using .env: ", found_dotenv)
if found_dotenv:
    dotenv.load_dotenv(found_dotenv)


class Client:

    def __init__(self, auth_url, api_base_url):
        self.auth_url = auth_url
        self.api_base_url = api_base_url
        self.authenticator = Authenticator('realnet',
                                              'http://localhost:8080',
                                              self.auth_url)

    def store_token(self, token):
        last_index = 0
        for chunk in self.chunks(token, 256):
            keyring.set_password("realnet", "access_token_{0}".format(chunk[0]), chunk[1])
            last_index = chunk[0]
        keyring.set_password("realnet", "access_token_count", last_index)

    def retrieve_token(self):
        count = keyring.get_password("realnet", "access_token_count")
        access_token = ''
        if count:
            for index in range(0, int(count) + 1):
                access_token += keyring.get_password("realnet", "access_token_{0}".format(index))
        else:
            access_token = self.authenticator.login()
            self.store_token(access_token)
        return access_token

    def chunks(self, s, n):
        for index, start in enumerate(range(0, len(s), n)):
            yield (index, s[start:start + n])

    def get_endpoint_url(self, path):
        return urljoin(self.api_base_url, path)

    def get(self, path, params):
        token = self.retrieve_token()
        headers = {}
        if token:
            headers = {"Authorization": "Bearer " + token}

        resp = requests.get(self.get_endpoint_url(path), headers=headers, params=params)

        if resp.status_code == requests.codes.ok:
            return resp.json()
        elif resp.status_code == 401:
            access_token = self.authenticator.login()
            self.store_token(access_token)
            return self.get(path, params)
        else:
            data = dump.dump_all(resp)
            print(data.decode('utf-8'))
            return None

    @classmethod
    def create(cls):
        return Client(os.getenv('AUTH_URL'), os.getenv('API_URL'))


